#' Summary on fragments
#'
#' This function outputs a summary on fragments using metrics generated by
#' Picard. Provides the number of mapped fragments, duplication rate and number
#' of unique fragments.
#'
#'
#' @param picard_list Named list of duplication metrics generated by Picard
#' as data frame. Data frames must be listed using `list()`.
#' To import Picard duplication metrics (.txt file) into R as data frame, use
#' `picard <- read.table("/path/to/picard/output", header = TRUE, fill = TRUE)`.
#' The list must be named e.g. `names(picard_list) <- c("name1","name2")`
#'
#' @return A table summarizing metrics on fragments.
#' @export
#'
#' @examples
#' data(CnT_H3K27ac_picard) # load example picard output
#' data(CnR_H3K27ac_picard) # load example picard output
#'
#' # To import Picard duplication metrics (.txt file) into R as data frame
#' # CnT_H3K27ac_picard <- read.table("/path/to/picard/output.txt", header = TRUE,fill = TRUE)
#'
#' picard <- list(CnT_H3K27ac_picard, CnR_H3K27ac_picard) # convert into list
#' names(picard) <- c("CnT_H3K27ac", "CnR_H3K27ac") # set names
#' df <- fragment_info(picard_list = picard)
#'
fragment_info <- function(picard_list){
  # check that list is named, if not, default names assigned
  picard_list <- check_list_names(picard_list)
  # create empty lists
  Mapped_Fragments <- c()
  Duplication_Rate <- c()
  Unique_Fragments <- c()
  # from picard summary retrieve the three metrics and store in a list
  for(i in seq_len(length(picard_list))){
    Mapped_Fragments <- c(Mapped_Fragments, picard_list[[i]]$READ_PAIRS_EXAMINED[1])
    Duplication_Rate <- c(Duplication_Rate, round(picard_list[[i]]$PERCENT_DUPLICATION[1]*100, 2))
    unique <- ((1-picard_list[[i]]$PERCENT_DUPLICATION[1]))*as.numeric(picard_list[[i]]$READ_PAIRS_EXAMINED[1])
    Unique_Fragments <- c(Unique_Fragments, round(unique))
  }
  # create data frame of metrics
  df_metric <- data.frame(Mapped_Fragments, Duplication_Rate, Unique_Fragments)
  rownames(df_metric) <- names(picard_list) # set row names as sample names
  return(df_metric)
}
